==> mockito-5.20.0.tar.gz: mockito-5.20.0/mockito-core/src/test/java/org/mockito/internal/stubbing/defaultanswers/ReturnsEmptyValuesTest.java <==
/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockito.internal.stubbing.defaultanswers;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeThat;
import static org.mockito.Mockito.mock;

import java.util.*;

import org.junit.Test;
import org.mockito.invocation.Invocation;
import org.mockitoutil.TestBase;

public class ReturnsEmptyValuesTest extends TestBase {

    private final ReturnsEmptyValues values = new ReturnsEmptyValues();

    @Test
    public void should_return_empty_collections_or_null_for_non_collections() {
        assertTrue(((Collection<?>) values.returnValueFor(Collection.class)).isEmpty());

        assertTrue(((Set<?>) values.returnValueFor(Set.class)).isEmpty());
        assertTrue(((SortedSet<?>) values.returnValueFor(SortedSet.class)).isEmpty());
        assertTrue(((HashSet<?>) values.returnValueFor(HashSet.class)).isEmpty());
        assertTrue(((TreeSet<?>) values.returnValueFor(TreeSet.class)).isEmpty());
        assertTrue(((LinkedHashSet<?>) values.returnValueFor(LinkedHashSet.class)).isEmpty());

        assertTrue(((List<?>) values.returnValueFor(List.class)).isEmpty());
        assertTrue(((ArrayList<?>) values.returnValueFor(ArrayList.class)).isEmpty());
        assertTrue(((LinkedList<?>) values.returnValueFor(LinkedList.class)).isEmpty());

        assertTrue(((Map<?, ?>) values.returnValueFor(Map.class)).isEmpty());
        assertTrue(((SortedMap<?, ?>) values.returnValueFor(SortedMap.class)).isEmpty());
        assertTrue(((HashMap<?, ?>) values.returnValueFor(HashMap.class)).isEmpty());
        assertTrue(((TreeMap<?, ?>) values.returnValueFor(TreeMap.class)).isEmpty());
        assertTrue(((LinkedHashMap<?, ?>) values.returnValueFor(LinkedHashMap.class)).isEmpty());

        assertNull(values.returnValueFor(String.class));
    }

    @Test
    public void should_return_empty_iterable() throws Exception {
        assertFalse(((Iterable<?>) values.returnValueFor(Iterable.class)).iterator().hasNext());
    }

    @Test
    public void should_return_primitive() {
        assertEquals(false, values.returnValueFor(Boolean.TYPE));
        assertEquals((char) 0, values.returnValueFor(Character.TYPE));
        assertEquals((byte) 0, values.returnValueFor(Byte.TYPE));
        assertEquals((short) 0, values.returnValueFor(Short.TYPE));
        assertEquals(0, values.returnValueFor(Integer.TYPE));
        assertEquals(0L, values.returnValueFor(Long.TYPE));
        assertEquals(0F, values.returnValueFor(Float.TYPE));
        assertEquals(0D, values.returnValueFor(Double.TYPE));
    }

    @Test
    public void should_return_non_zero_for_compareTo_method() {
        //
        // given
        Date d = mock(Date.class);
        d.compareTo(new Date());
        Invocation compareTo = this.getLastInvocation();

        // when
        Object result = values.answer(compareTo);

        // then
        assertTrue(result != (Object) 0);
    }

    @SuppressWarnings("SelfComparison")
    @Test
    public void should_return_zero_if_mock_is_compared_to_itself() {
        // given
        Date d = mock(Date.class);
        d.compareTo(d);
        Invocation compareTo = this.getLastInvocation();

        // when
        Object result = values.answer(compareTo);

        // then
        assertEquals(0, result);
    }

    @Test
    public void should_return_empty_Optional() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.Stream", "java.util.Optional");
    }

    @Test
    public void should_return_empty_OptionalDouble() throws Exception {
        verify_empty_Optional_is_returned(
                "java.util.stream.DoubleStream", "java.util.OptionalDouble");
    }

    @Test
    public void should_return_empty_OptionalInt() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.IntStream", "java.util.OptionalInt");
    }

    @Test
    public void should_return_empty_OptionalLong() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.LongStream", "java.util.OptionalLong");
    }

    private void verify_empty_Optional_is_returned(String streamFqcn, String optionalFqcn)
            throws Exception {
        // given
        assumeThat("JDK 8+ required for Optional", isJavaVersionAtLeast(8), is(true));

        Class<?> streamType = Class.forName(streamFqcn);

        Object stream = mock(streamType);
        Object optional = streamType.getMethod("findAny").invoke(stream);
        assertNotNull(optional);
        assertFalse((Boolean) Class.forName(optionalFqcn).getMethod("isPresent").invoke(optional));

        Invocation findAny = this.getLastInvocation();

        // when
        Object result = values.answer(findAny);

        // then
        assertEquals(optional, result);
    }

    @Test
    public void should_return_empty_Stream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.Stream");
    }

    @Test
    public void should_return_empty_DoubleStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.DoubleStream");
    }

    @Test
    public void should_return_empty_IntStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.IntStream");
    }

    @Test
    public void should_return_empty_LongStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.LongStream");
    }

    private void verify_empty_Stream_is_returned(String streamFqcn) throws Exception {
        // given
        assumeThat("JDK 8+ required for Stream", isJavaVersionAtLeast(8), is(true));

        Class<?> streamType = Class.forName(streamFqcn);

        // when
        Object stream = values.returnValueFor(streamType);
        long count = (Long) streamType.getMethod("count").invoke(stream);

        // then
        assertEquals("count of empty " + streamFqcn, 0L, count);
    }

    @Test
    public void should_return_empty_duration() throws Exception {
        // given
        assumeThat("JDK 8+ required for Duration", isJavaVersionAtLeast(8), is(true));
        final String fqcn = "java.time.Duration";
        Class<?> durationClass = Class.forName(fqcn);

        // when
        final Object duration = values.returnValueFor(durationClass);
        final int nano = (Integer) durationClass.getMethod("getNano").invoke(duration);
        final long seconds = (Long) durationClass.getMethod("getSeconds").invoke(duration);

        // then
        assertEquals("nano of empty " + fqcn, 0, nano);
        assertEquals("seconds of empty " + fqcn, 0L, seconds);
    }

    @Test
    public void should_return_empty_sequenced_collection() throws Exception {
        assumeThat("JDK 21+ required for SequencedCollection", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedCollectionClass = Class.forName("java.util.SequencedCollection");

        Object result = values.returnValueFor(sequencedCollectionClass);
        assertNotNull("SequencedCollection should return non-null value", result);
        assertTrue("Should return empty collection", ((Collection<?>) result).isEmpty());
        assertTrue("Should return ArrayList instance", result instanceof ArrayList);
    }

    @Test
    public void should_return_empty_sequenced_set() throws Exception {
        assumeThat("JDK 21+ required for SequencedSet", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedSetClass = Class.forName("java.util.SequencedSet");

        Object result = values.returnValueFor(sequencedSetClass);
        assertNotNull("SequencedSet should return non-null value", result);
        assertTrue("Should return empty set", ((Set<?>) result).isEmpty());
        assertTrue("Should return LinkedHashSet instance", result instanceof LinkedHashSet);
    }

    @Test
    public void should_return_empty_sequenced_map() throws Exception {
        assumeThat("JDK 21+ required for SequencedMap", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedMapClass = Class.forName("java.util.SequencedMap");

        Object result = values.returnValueFor(sequencedMapClass);
        assertNotNull("SequencedMap should return non-null value", result);
        assertTrue("Should return empty map", ((Map<?, ?>) result).isEmpty());
        assertTrue("Should return LinkedHashMap instance", result instanceof LinkedHashMap);
    }

    /**
     * Checks if the current Java version is at least the specified version.
     */
    private boolean isJavaVersionAtLeast(int majorVersion) {
        String javaVersion = System.getProperty("java.version");
        String[] versionParts = javaVersion.split("\\.");
        int currentMajorVersion = Integer.parseInt(versionParts[0]);
        return currentMajorVersion >= majorVersion;
    }
}

==> mockito-5.21.0.tar.gz: mockito-5.21.0/mockito-core/src/test/java/org/mockito/internal/stubbing/defaultanswers/ReturnsEmptyValuesTest.java <==
/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockito.internal.stubbing.defaultanswers;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;
import static org.junit.Assume.assumeThat;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.mockito.Mockito.mock;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CompletionStage;
import java.util.concurrent.Future;

import org.junit.Test;
import org.mockito.invocation.Invocation;
import org.mockitoutil.TestBase;

public class ReturnsEmptyValuesTest extends TestBase {

    private final ReturnsEmptyValues values = new ReturnsEmptyValues();

    @Test
    public void should_return_empty_collections_or_null_for_non_collections() {
        assertTrue(((Collection<?>) values.returnValueFor(Collection.class)).isEmpty());

        assertTrue(((Set<?>) values.returnValueFor(Set.class)).isEmpty());
        assertTrue(((SortedSet<?>) values.returnValueFor(SortedSet.class)).isEmpty());
        assertTrue(((HashSet<?>) values.returnValueFor(HashSet.class)).isEmpty());
        assertTrue(((TreeSet<?>) values.returnValueFor(TreeSet.class)).isEmpty());
        assertTrue(((LinkedHashSet<?>) values.returnValueFor(LinkedHashSet.class)).isEmpty());

        assertTrue(((List<?>) values.returnValueFor(List.class)).isEmpty());
        assertTrue(((ArrayList<?>) values.returnValueFor(ArrayList.class)).isEmpty());
        assertTrue(((LinkedList<?>) values.returnValueFor(LinkedList.class)).isEmpty());

        assertTrue(((Map<?, ?>) values.returnValueFor(Map.class)).isEmpty());
        assertTrue(((SortedMap<?, ?>) values.returnValueFor(SortedMap.class)).isEmpty());
        assertTrue(((HashMap<?, ?>) values.returnValueFor(HashMap.class)).isEmpty());
        assertTrue(((TreeMap<?, ?>) values.returnValueFor(TreeMap.class)).isEmpty());
        assertTrue(((LinkedHashMap<?, ?>) values.returnValueFor(LinkedHashMap.class)).isEmpty());

        assertNull(values.returnValueFor(String.class));
    }

    @Test
    public void should_return_empty_iterable() throws Exception {
        assertFalse(((Iterable<?>) values.returnValueFor(Iterable.class)).iterator().hasNext());
    }

    @Test
    public void should_return_primitive() {
        assertEquals(false, values.returnValueFor(Boolean.TYPE));
        assertEquals((char) 0, values.returnValueFor(Character.TYPE));
        assertEquals((byte) 0, values.returnValueFor(Byte.TYPE));
        assertEquals((short) 0, values.returnValueFor(Short.TYPE));
        assertEquals(0, values.returnValueFor(Integer.TYPE));
        assertEquals(0L, values.returnValueFor(Long.TYPE));
        assertEquals(0F, values.returnValueFor(Float.TYPE));
        assertEquals(0D, values.returnValueFor(Double.TYPE));
    }

    @Test
    public void should_return_non_zero_for_compareTo_method() {
        //
        // given
        Date d = mock(Date.class);
        d.compareTo(new Date());
        Invocation compareTo = this.getLastInvocation();

        // when
        Object result = values.answer(compareTo);

        // then
        assertTrue(result != (Object) 0);
    }

    @SuppressWarnings("SelfComparison")
    @Test
    public void should_return_zero_if_mock_is_compared_to_itself() {
        // given
        Date d = mock(Date.class);
        d.compareTo(d);
        Invocation compareTo = this.getLastInvocation();

        // when
        Object result = values.answer(compareTo);

        // then
        assertEquals(0, result);
    }

    @Test
    public void should_return_empty_Optional() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.Stream", "java.util.Optional");
    }

    @Test
    public void should_return_empty_OptionalDouble() throws Exception {
        verify_empty_Optional_is_returned(
                "java.util.stream.DoubleStream", "java.util.OptionalDouble");
    }

    @Test
    public void should_return_empty_OptionalInt() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.IntStream", "java.util.OptionalInt");
    }

    @Test
    public void should_return_empty_OptionalLong() throws Exception {
        verify_empty_Optional_is_returned("java.util.stream.LongStream", "java.util.OptionalLong");
    }

    private void verify_empty_Optional_is_returned(String streamFqcn, String optionalFqcn)
            throws Exception {
        // given
        assumeThat("JDK 8+ required for Optional", isJavaVersionAtLeast(8), is(true));

        Class<?> streamType = Class.forName(streamFqcn);

        Object stream = mock(streamType);
        Object optional = streamType.getMethod("findAny").invoke(stream);
        assertNotNull(optional);
        assertFalse((Boolean) Class.forName(optionalFqcn).getMethod("isPresent").invoke(optional));

        Invocation findAny = this.getLastInvocation();

        // when
        Object result = values.answer(findAny);

        // then
        assertEquals(optional, result);
    }

    @Test
    public void should_return_empty_Stream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.Stream");
    }

    @Test
    public void should_return_empty_DoubleStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.DoubleStream");
    }

    @Test
    public void should_return_empty_IntStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.IntStream");
    }

    @Test
    public void should_return_empty_LongStream() throws Exception {
        verify_empty_Stream_is_returned("java.util.stream.LongStream");
    }

    private void verify_empty_Stream_is_returned(String streamFqcn) throws Exception {
        // given
        assumeThat("JDK 8+ required for Stream", isJavaVersionAtLeast(8), is(true));

        Class<?> streamType = Class.forName(streamFqcn);

        // when
        Object stream = values.returnValueFor(streamType);
        long count = (Long) streamType.getMethod("count").invoke(stream);

        // then
        assertEquals("count of empty " + streamFqcn, 0L, count);
    }

    @Test
    public void should_return_empty_duration() throws Exception {
        // given
        assumeThat("JDK 8+ required for Duration", isJavaVersionAtLeast(8), is(true));
        final String fqcn = "java.time.Duration";
        Class<?> durationClass = Class.forName(fqcn);

        // when
        final Object duration = values.returnValueFor(durationClass);
        final int nano = (Integer) durationClass.getMethod("getNano").invoke(duration);
        final long seconds = (Long) durationClass.getMethod("getSeconds").invoke(duration);

        // then
        assertEquals("nano of empty " + fqcn, 0, nano);
        assertEquals("seconds of empty " + fqcn, 0L, seconds);
    }

    @Test
    public void should_return_empty_sequenced_collection() throws Exception {
        assumeThat("JDK 21+ required for SequencedCollection", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedCollectionClass = Class.forName("java.util.SequencedCollection");

        Object result = values.returnValueFor(sequencedCollectionClass);
        assertNotNull("SequencedCollection should return non-null value", result);
        assertTrue("Should return empty collection", ((Collection<?>) result).isEmpty());
        assertTrue("Should return ArrayList instance", result instanceof ArrayList);
    }

    @Test
    public void should_return_empty_sequenced_set() throws Exception {
        assumeThat("JDK 21+ required for SequencedSet", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedSetClass = Class.forName("java.util.SequencedSet");

        Object result = values.returnValueFor(sequencedSetClass);
        assertNotNull("SequencedSet should return non-null value", result);
        assertTrue("Should return empty set", ((Set<?>) result).isEmpty());
        assertTrue("Should return LinkedHashSet instance", result instanceof LinkedHashSet);
    }

    @Test
    public void should_return_empty_sequenced_map() throws Exception {
        assumeThat("JDK 21+ required for SequencedMap", isJavaVersionAtLeast(21), is(true));

        Class<?> sequencedMapClass = Class.forName("java.util.SequencedMap");

        Object result = values.returnValueFor(sequencedMapClass);
        assertNotNull("SequencedMap should return non-null value", result);
        assertTrue("Should return empty map", ((Map<?, ?>) result).isEmpty());
        assertTrue("Should return LinkedHashMap instance", result instanceof LinkedHashMap);
    }

    /**
     * Checks if the current Java version is at least the specified version.
     */
    private boolean isJavaVersionAtLeast(int majorVersion) {
        String javaVersion = System.getProperty("java.version");
        String[] versionParts = javaVersion.split("\\.");
        int currentMajorVersion = Integer.parseInt(versionParts[0]);
        return currentMajorVersion >= majorVersion;
    }

    interface ReturnFuture {
        Future<String> f();
    }

    interface ReturnCompletableFuture {
        CompletableFuture<Void> v();
    }

    interface ReturnCompletionStage {
        CompletionStage<Integer> s();
    }

    @Test
    public void returnsCompletedFuture_forFuture() throws Exception {
        ReturnFuture m = mock(ReturnFuture.class);
        Future<String> fut = m.f();
        assertNotNull(fut);
        assertTrue(fut.isDone());
        assertNull(fut.get());
    }

    @Test
    public void returnsCompletedFuture_forCompletableFuture() {
        ReturnCompletableFuture m = mock(ReturnCompletableFuture.class);
        CompletableFuture<Void> fut = m.v();
        assertNotNull(fut);
        assertTrue(fut.isDone());
        assertDoesNotThrow(fut::join);
    }

    @Test
    public void returnsCompletedFuture_forCompletionStage() {
        ReturnCompletionStage m = mock(ReturnCompletionStage.class);
        CompletionStage<Integer> st = m.s();
        assertNotNull(st);
        assertTrue(st.toCompletableFuture().isDone());
        assertNull(st.toCompletableFuture().join());
    }
}
